@@ -646,7 +646,12 @@ SEXP processBlock(T * const sobj) {
     SET_S4_OBJECT(obj);
     // SET_OBJECT(obj, 1); // this flag seems kind of pointless
   }
-  return obj;
+  if(TYPEOF(obj) == PROMSXP) {
+    Rcpp::warning("PROMSXP detected, replacing with NULL (see https://github.com/traversc/qs/issues/93)");
+    return R_NilValue;
+  } else {
+    return obj;
+  }
 }
 
 
