@@ -1,19 +1,19 @@
 /* qs - Quick Serialization of R Objects
  Copyright (C) 2019-present Travers Ching
- 
+
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.
- 
+
  You can contact the author at:
  https://github.com/traversc/qs
  */
@@ -55,28 +55,28 @@
 //   }
 // };
 
-template <class decompress_env> 
+template <class decompress_env>
 struct Data_Thread_Context {
   std::ifstream & myFile;
   decompress_env denv; // default constructor
   const unsigned int nthreads;
-  
+
   uint64_t blocks_total;
   std::atomic<uint64_t> blocks_read;
   std::atomic<uint64_t>  blocks_processed;
-  
+
   std::vector<bool> primary_block = std::vector<bool>(nthreads, true);
   std::vector< std::vector<char> > zblocks; // one per thread
   std::vector< std::vector<char> > data_blocks; // one per thread
   std::vector< std::vector<char> > data_blocks2; // one per thread
   std::pair<char*, uint64_t> data_pass; // default constructor
-  
+
   std::vector< std::atomic<char*> > block_pointers;
   std::vector< std::atomic<uint64_t> > block_sizes;
   std::vector< std::atomic<uint8_t> > data_task;
   std::vector<std::thread> threads;
 
-  Data_Thread_Context(std::ifstream & mf, unsigned int nt, QsMetadata qm) : 
+  Data_Thread_Context(std::ifstream & mf, unsigned int nt, QsMetadata qm) :
     myFile(mf), nthreads(nt), blocks_total(qm.clength), blocks_read(0), blocks_processed(0),
     zblocks(std::vector< std::vector<char> >(nt, std::vector<char>(this->denv.compressBound(BLOCKSIZE)))),
     data_blocks(std::vector< std::vector<char> >(nt, std::vector<char>(BLOCKSIZE))),
@@ -97,7 +97,7 @@ struct Data_Thread_Context {
       threads.push_back(std::thread(&Data_Thread_Context::worker_thread, this, i));
     }
   }
-  
+
   void worker_thread(unsigned int thread_id) {
     std::array<char,4> zsize_ar;
     for(uint64_t i=thread_id; i < blocks_total; i += nthreads) {
@@ -109,7 +109,7 @@ struct Data_Thread_Context {
       uint32_t zsize = unaligned_cast<uint32_t>(zsize_ar.data(),0);
       myFile.read(zblocks[thread_id].data(), zsize);
       blocks_read++;
-      
+
       // task marching orders from main thread
       // 0 = wait
       // 1 = nothing (main thread will use block as is)
@@ -141,12 +141,12 @@ struct Data_Thread_Context {
         data_task[thread_id] = 0;
       }
       // }
-      
+
       primary_block[thread_id] = !primary_block[thread_id];
     }
     // tout << thread_id << " finished thread for loop\n" << std::flush;
   }
-  
+
   void finish() {
     blocks_processed++;
     for(unsigned int i=0; i < nthreads; i++) {
@@ -154,7 +154,7 @@ struct Data_Thread_Context {
       threads[i].join();
     }
   }
-  
+
   std::pair<char*, uint64_t> get_block_ptr() {
     uint64_t current_block = blocks_processed % nthreads;
     blocks_processed++;
@@ -165,7 +165,7 @@ struct Data_Thread_Context {
     uint64_t temp_size = data_pass.second;
     return std::pair<char*, uint64_t>(temp_ptr, temp_size);
   }
-  
+
   void decompress_data_direct(char* bpointer) {
     uint64_t current_block = blocks_processed % nthreads;
     blocks_processed++;
@@ -176,7 +176,7 @@ struct Data_Thread_Context {
   }
 };
 
-template <class decompress_env> 
+template <class decompress_env>
 struct Data_Context_MT {
   QsMetadata qm;
   std::ifstream & myFile;
@@ -184,14 +184,13 @@ struct Data_Context_MT {
   xxhash_env xenv;
   std::unordered_map<uint32_t, SEXP> object_ref_hash;
   bool use_alt_rep_bool;
-  
+
   std::vector<uint8_t> shuffleblock = std::vector<uint8_t>(256);
   char* block_data;
   uint64_t block_size = 0;
   uint64_t data_offset = 0;
-  std::string temp_string = std::string(256, '\0');
-  
-  Data_Context_MT(std::ifstream & mf, QsMetadata qm, bool use_alt_rep, unsigned int nthreads) : 
+
+  Data_Context_MT(std::ifstream & mf, QsMetadata qm, bool use_alt_rep, unsigned int nthreads) :
     qm(qm), myFile(mf), dtc(mf, nthreads-1, qm), use_alt_rep_bool(use_alt_rep) {}
   void readHeader(qstype & object_type, uint64_t & r_array_len) {
     if(data_offset >= block_size) decompress_block();
@@ -242,6 +241,19 @@ struct Data_Context_MT {
       }
     }
   }
+  char * tempBlock(uint64_t data_size) {
+    if(data_size > shuffleblock.size()) shuffleblock.resize(data_size);
+    return reinterpret_cast<char*>(shuffleblock.data());
+  }
+  char * tempBlock() {
+    return reinterpret_cast<char*>(shuffleblock.data());
+  }
+  std::string getString(uint64_t data_size) {
+    std::string temp_string;
+    temp_string.resize(data_size);
+    getBlockData(&temp_string[0], data_size);
+    return temp_string;
+  }
   void getShuffleBlockData(char* outp, uint64_t data_size, uint64_t bytesoftype) {
     if(data_size >= MIN_SHUFFLE_ELEMENTS) {
       if(data_size > shuffleblock.size()) shuffleblock.resize(data_size);
